from datetime import date

import pytest
import requests
from fr_date import conv

from seuils.usure import local_liens, source

data = local_liens()
param_list = [(x, data[x]) for x in data]


def test_remote():
    response = requests.get(f"{source}seuils.json")
    assert response.status_code == 200
    response = requests.get(f"{source}avis.json")
    assert response.status_code == 200


@pytest.mark.parametrize(
    "a, b",
    param_list,
)
def test_lien_fonctionnel(a, b):
    assert type(conv(a, True)) is date
    response = requests.get(b)
    assert response.status_code == 200


def test_lien_unique():
    data = local_liens()
    liste = [data[x] for x in data]
    assert len(liste) == len(set(liste))



import sys
from datetime import date
from importlib import resources

from jinja2 import Template

from . import assets
from .figures import df, graphique


def pages(filepath):
    fig = graphique(df(1500, 5000, 10000))
    output_html_path = filepath
    input_html_path = resources.files(assets) / "gitlab_page_template.html"
    plotly_jinja_data = {
        "fig": fig.to_html(full_html=False),
        "maj": date.today().isoformat(),
    }
    with open(output_html_path, "w", encoding="utf-8") as output_file:
        with open(input_html_path) as template_file:
            j2_template = Template(template_file.read())
            output_file.write(j2_template.render(plotly_jinja_data))


if __name__ == "__main__":
    pages(sys.argv[1])



from datetime import date, timedelta

from seuils.usure import get_trimestre, local_liens


def daterange(start_date: date, end_date: date):
    days = int((end_date - start_date).days)
    for n in range(days):
        yield start_date + timedelta(n)


def test_historique():
    start_date = date(2005, 1, 1)
    end_date = date.today()
    avis = local_liens()
    for single_date in daterange(start_date, end_date):
        assert type(avis[get_trimestre(single_date)]) is str



import json
import os
from datetime import date
from decimal import Decimal
from importlib import resources

import requests
from fr_date import conv

from . import data, pdf

source = (
    "https://seuils-usure-outils-jcp-1feb902fc0837a7803cd3e9a229a5b9fc188f66.gitlab.io/"
)


def local_data():
    with open(resources.files(data) / "seuils.json", "r") as f:
        seuils = json.loads(f.read())
    return seuils


def remote_data():
    try:
        seuils = requests.get(f"{source}seuils.json").json()
    except requests.exceptions.ConnectionError:
        seuils = local_data()
    return seuils


def local_liens():
    with open(resources.files(data) / "avis.json", "r") as f:
        avis = json.loads(f.read())
    return avis


def remote_liens():
    try:
        avis = requests.get(f"{source}avis.json").json()
    except requests.exceptions.ConnectionError:
        avis = local_liens()
    return avis


def local_pdf(jour):
    file = resources.files(pdf) / f"{jour}.pdf"
    if os.path.isfile(file):
        return file
    return False


def remote_pdf(jour):
    try:
        avis = requests.get(f"{source}assets/pdf/{jour}.pdf")
    except requests.exceptions.ConnectionError:
        avis = local_liens()
    return avis


def get_trimestre(jour):
    if type(jour) is date:
        vigueur = jour
    else:
        vigueur = conv(jour, True)
        if type(vigueur) is not date:
            raise ValueError
    if vigueur.year == 2023:
        return vigueur.replace(day=1).isoformat()
    else:
        mois = {}
        for m in range(1, 13):
            mois[m] = m - (m - 1) % 3
        return vigueur.replace(month=mois[vigueur.month], day=1).isoformat()


def get_lien(jour):
    trimestre = get_trimestre(jour)
    avis = local_liens()
    if trimestre in avis:
        lien = avis[trimestre]
    else:
        avis = remote_liens()
        if trimestre in avis:
            lien = avis[trimestre]
        else:
            lien = "lien introuvable"
    return lien


def get_pdf(jour):
    trimestre = get_trimestre(jour)
    avis = local_pdf(trimestre)
    if avis:
        return avis
    else:
        avis = remote_pdf(trimestre)
        if avis:
            return avis
        else:
            return None


def get_taux(jour, montant=None, categorie=None):
    trimestre = get_trimestre(jour)
    data = local_data()
    if trimestre in data:
        seuils = data[trimestre]["seuils"]
    else:
        data = remote_data()
        if trimestre in data:
            seuils = data[trimestre]["seuils"]
        else:
            return None
    if montant:
        for s in seuils:
            if Decimal(s["min"]) < montant <= Decimal(s["max"]):
                if categorie and "categorie" in s:
                    if categorie == s["categorie"]:
                        return Decimal(s["taux"])
                elif "categorie" in s:
                    return seuils
                else:
                    return Decimal(s["taux"])
    return seuils



import pandas as pd
import plotly

from seuils.figures import df, graphique


def test_dataframe():
    data = [df(1000), df(4000), df(10000)]
    for d in data:
        assert type(d) is pd.DataFrame
        assert len(d) >= 87


def test_fig():
    g = graphique(df(3000))
    assert type(g) is plotly.graph_objs._figure.Figure



import requests

from seuils.usure import local_liens, local_pdf, source


def test_remote():
    for x in local_liens():
        response = requests.get(f"{source}/assets/pdf/{x}.pdf")
        assert response.status_code == 200


def test_local():
    for x in local_liens():
        assert local_pdf(x) is not False



import pandas as pd
import plotly.express as px
from fr_date import conv

from . import usure

dates = sorted([conv(x, True) for x in usure.local_liens().keys()])


def df(*montants):
    a, b, c = [], [], []
    for montant in montants:
        a += dates
        b += [
            usure.get_taux(jour=x, montant=montant, categorie="classique")
            for x in dates
        ]
        c += [montant for x in dates]

    data = {"Entrée en vigueur": a, "Taux d'usure (en %)": b, "Montant": c}
    dataframe = pd.DataFrame(data)
    return dataframe


def graphique(data):
    fig = px.line(data, x="Entrée en vigueur", y="Taux d'usure (en %)", color="Montant")
    return fig
